/************************************************************************/
/* dbutil.c - 								*/
/************************************************************************/
/* dbutil
 * Copyright (C) 1999-2001  Todd Shadburn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/* (04-24-2001) Convert to using GString for the SQL statements.Todd.	*/
/* (04-14-2001) Moved the GTK specific functions to gtkdbutil.c Todd	*/
/* (01-05-2000) Added str variable to dbutil_table_sync_with_gui because*/
/*		gtk_entry_get_text() returns a empty string("") instead	*/
/*		of a NULL when a GtkEntry is blank. Todd		*/
/* (01-04-2000) Some cleanup, added table_get_where_fields() and moved	*/
/*		the GTK functions back into the API. Todd.		*/
/* (01-03-2000) Working version of fbutil_relationset_select_new().Todd	*/
/* (12-14-1999) Original creation date.Todd Shadburn			*/
/************************************************************************/

#include <glib.h>
#include "dbutil.h"

#define SQLSTRLEN	256

/************************************************************************/
/* Forward declarations							*/
/************************************************************************/
gint table_is_cleared(dbtable_t *dbtable);
GString * table_get_where_fields(dbtable_t *table);
GString * field_get_where(dbfield_t *field);
GString * relation_get_foreign_where(dbrelation_t *relation);
GString * field_get_sql_value(dbfield_t *field);
sqlhandle_t *sqlhandle_new(void);
gint table_used_in_browser(dbbrowser_t *dbbrowser, dbtable_t *dbtable);

/************************************************************************/
/* public function prototypes						*/
/************************************************************************/
dbtable_t *
dbutil_table_new(gchar *tablename)
	{
	dbtable_t *table;
	if(!tablename)
		return(NULL);
	table=(dbtable_t *)g_malloc(sizeof(dbtable_t));
	if(!table)
		return(NULL);
	table->tablename = g_strdup(tablename);
	table->fieldset = NULL;
	table->data = NULL;
	table->flags = 0;
	table->widget = NULL;
	return(table);
	}

void
dbutil_table_free(dbtable_t *dbtable)
	{
	dbfield_t *field;
	dbfield_t *nfield;
	if(!dbtable)
		return;
	if(dbtable->tablename)
		g_free(dbtable->tablename);
	field=dbtable->fieldset;
	while(field)
		{
		nfield=field->next;
		field->next=NULL;
		field=nfield;
		}
	g_free(dbtable);
	dbtable=NULL;
	}

gint
dbutil_table_add_field(dbtable_t *dbtable, dbfield_t *dbfield)
	{
	dbfield_t *field;
	if(!dbtable || !dbfield)
		return(-1);
	if(dbutil_table_get_field(dbtable, dbfield->dbfield))
		return(-2);
	dbfield->table = dbtable;
	field = dbtable->fieldset;
	if(field)
		{
		while(field->next)
			field=field->next;
		field->next = dbfield;
		}
	else
		dbtable->fieldset = dbfield;
	return(0);
	}

dbfield_t *
dbutil_table_get_primary_field(dbtable_t *dbtable)
	{
	dbfield_t *field;
	if(!dbtable)
		return(NULL);
	field = dbtable->fieldset;
	while(field)
		{
		if(field->flags & DBF_PRIMARYKEY)
			return(field);
		field = field->next;
		}
	return(NULL);
	}

dbfield_t *
dbutil_table_get_field(dbtable_t *dbtable, gchar *fieldname)
	{
	dbfield_t *field;
	if(!dbtable || !fieldname)
		return(NULL);
	field = dbtable->fieldset;
	while(field)
		{
		if(!g_strcasecmp(field->dbfield, fieldname))
			return(field);
		field = field->next;
		}
	return(NULL);
	}

int
dbutil_table_required_fields_filled(dbtable_t * dbtable)
	{
	dbfield_t *field;
	if(!dbtable)
		return(0);
	field = dbtable->fieldset;
	while(field)
		{
		if(field->flags & DBF_REQUIRED)
			{
			if(!(field->flags & DBF_HASDATA))
				return(0);
			}
		field = field->next;
		}
	return(1);
	}

void
dbutil_table_set_flags(dbtable_t *dbtable, gint flags)
	{
	if(!dbtable)
		return;
	dbtable->flags |= flags;
	return;
	}

void
dbutil_table_clear_flags(dbtable_t *dbtable, gint flags)
	{
	if(!dbtable)
		return;
	dbtable->flags -= (dbtable->flags & flags);
	return;
	}

void
dbutil_table_set_widget(dbtable_t *dbtable, gpointer widget)
	{
	if(!dbtable || !widget)
		return;
	dbtable->widget = widget;
	dbtable->flags |= DBT_MULTIROW;
	return;
	}

/************************************************************************/
dbtableset_t *
dbutil_tableset_new(gchar *tablesetname)
	{
	dbtableset_t *tableset;
	if(!tablesetname)
		return(NULL);
	tableset=(dbtableset_t *)g_malloc(sizeof(dbtableset_t));
	if(!tableset)
		return(NULL);
	tableset->tablesetname=g_strdup(tablesetname);
	tableset->tablelist = g_slist_alloc();
	tableset->data=NULL;
	tableset->flags=0;
	return(tableset);
	}

void
dbutil_tableset_free(dbtableset_t *dbtableset)
	{
	if(!dbtableset)
		return;
	if(dbtableset->tablesetname)
		g_free(dbtableset->tablesetname);
	if(dbtableset->tablelist)
		{
		g_slist_free(dbtableset->tablelist);
		}
	g_free(dbtableset);
	dbtableset=NULL;
	return;
	}

int
dbutil_tableset_add_table(dbtableset_t *dbtableset, dbtable_t *dbtable)
	{
	dbtable_t *table;
	GSList *list;
	if(!dbtableset)
		return(-1);
	list=dbtableset->tablelist;
	while(list)
		{
		if( (table=(dbtable_t *)list->data) )
			if(!g_strcasecmp(table->tablename, dbtable->tablename))
				return(-1);
		list=list->next;
		}
	g_slist_append(dbtableset->tablelist, dbtable);
	return(0);
	}

dbtable_t *
dbutil_tableset_get_table(dbtableset_t *dbtableset, gchar *tablename)
	{
	dbtable_t *table;
	GSList *list;
	if(!dbtableset)
		return(NULL);
	list=dbtableset->tablelist;
	while(list)
		{
		if( (table=(dbtable_t *)list->data) )
			if(!g_strcasecmp(table->tablename, tablename))
				return(table);
		list=list->next;
		}
	return(NULL);
	}


/************************************************************************/
dbfield_t *
dbutil_field_new(void *widget, gchar *field)
	{
	dbfield_t *dbfield;
	if(!field)
		return(NULL);
	dbfield = (dbfield_t *)g_malloc(sizeof(dbfield_t));
	if(!dbfield)
		return(NULL);
	dbfield->table = NULL;
	dbfield->dbfield = g_strdup(field);
	dbfield->flags = 0;
	dbfield->widget = widget;
	dbfield->data = NULL;
	dbfield->next = NULL;
	return(dbfield);
	}


void
dbutil_field_free(dbfield_t *dbfield)
	{
	if(!dbfield)
		return;
	if(dbfield->dbfield) g_free(dbfield->dbfield);
	g_free(dbfield);
	dbfield = NULL;
	return;
	}


void
dbutil_field_set_data(dbfield_t *dbfield, gpointer data)
	{
	if(!dbfield)
		return;
	dbfield->data = g_strdup((gchar *)data);
	dbfield->flags |= DBF_HASDATA;
	return;
	}


void
dbutil_field_clear_data(dbfield_t *dbfield)
	{
	if(!dbfield)
		return;
	if(dbfield->data) g_free(dbfield->data);
	dbfield->data = NULL;
	dbfield->flags -= (dbfield->flags & DBF_HASDATA);
	return;
	}


void
dbutil_field_set_primary(dbfield_t *dbfield)
	{
	if(!dbfield)
		return;
	dbfield->flags |= DBF_PRIMARYKEY;
	return;
	}

void
dbutil_field_set_flags(dbfield_t *dbfield, gint flags)
	{
	if(!dbfield)
		return;
	dbfield->flags |= flags;
	return;
	}

void
dbutil_field_clear_flags(dbfield_t *dbfield, gint flags)
	{
	if(!dbfield)
		return;
	dbfield->flags -= (dbfield->flags & flags);
	return;
	}


/************************************************************************/
dbrelation_t *
dbutil_relation_new(gchar *name, dbfield_t *primary, dbfield_t *foreign)
	{
	dbrelation_t *dbrelation;
	if(!name || !primary || !foreign)
		return(NULL);
	dbrelation = (dbrelation_t *)g_malloc(sizeof(dbrelation_t));
	if(!dbrelation)
		return(NULL);
	dbrelation->relationname = g_strdup(name);
	dbrelation->primary = primary;
	dbrelation->foreign = foreign;
	dbrelation->data = NULL;
	dbrelation->flags = 0;
	return(dbrelation);
	}


void
dbutil_relation_free(dbrelation_t *dbrelation)
	{
	if(!dbrelation)
		return;
	if(dbrelation->relationname) g_free(dbrelation->relationname);
	g_free(dbrelation);
	dbrelation = NULL;
	return;
	}


void
dbutil_relation_set_data(dbrelation_t *dbrelation, gpointer data)
	{
	if(!dbrelation)
		return;
	dbrelation->data = data;
	return;
	}

/************************************************************************/
dbrelationset_t *
dbutil_relationset_new(gchar *relationsetname, dbtableset_t *tableset)
	{
	dbrelationset_t *rs;
	if(!relationsetname || !tableset)
		return(NULL);
	rs=(dbrelationset_t *)g_malloc(sizeof(dbrelationset_t));
	if(!rs)
		return(NULL);
	rs->relationsetname=g_strdup(relationsetname);
	rs->tableset=tableset;
	rs->relationlist=g_slist_alloc();
	rs->data=NULL;
	rs->flags=0;
	return(rs);
	}

void
dbutil_relationset_free(dbrelationset_t *dbrelationset)
	{
	if(!dbrelationset)
		return;
	if(dbrelationset->relationsetname)
		g_free(dbrelationset->relationsetname);
	if(dbrelationset->relationlist)
		{
		g_slist_free(dbrelationset->relationlist);
		}
	g_free(dbrelationset);
	dbrelationset=NULL;
	return;
	}


void
dbutil_relationset_full_free(dbrelationset_t *dbrelationset)
	{
	dbrelation_t *relation;
	dbtable_t *table;
	dbfield_t *field;
	dbfield_t *nfield;
	GSList *list;
	if(!dbrelationset)
		return;
	if(dbrelationset->relationsetname)
		g_free(dbrelationset->relationsetname);
	if(dbrelationset->relationlist)
		{
		list=dbrelationset->relationlist;
		while(list)
			{
			if( (relation=(dbrelation_t *)list->data) )
				dbutil_relation_free(relation);
			list=list->next;
			}
		}
	if(dbrelationset->tableset)
		{
		list=dbrelationset->tableset->tablelist;
		while(list)
			{
			if( (table=(dbtable_t *)list->data) )
				{
				field=table->fieldset;
				while(field)
					{
					nfield=field->next;
					dbutil_field_free(field);
					field=nfield;
					}
				dbutil_table_free(table);
				}
			list=list->next;
			}
		dbutil_tableset_free(dbrelationset->tableset);
		}
	g_free(dbrelationset);
	dbrelationset=NULL;
	return;
	}


int
dbutil_relationset_add_relation(dbrelationset_t *dbrelationset,
				dbrelation_t *dbrelation)
	{
	GSList *list;
	dbrelation_t *relation;
	if(!dbrelationset)
		return(-1);
	list=dbrelationset->relationlist;
	while(list)
		{
		if( (relation=(dbrelation_t *)list->data) )
		if(!g_strcasecmp(relation->relationname, dbrelation->relationname))
			return(-2);
		list=list->next;
		}
	g_slist_append(dbrelationset->relationlist, dbrelation);
	return(0);
	}


/************************************************************************/
/************************************************************************/
GString *
dbutil_relationset_select(dbrelationset_t *relationset,
				gchar *primarytable, gint userelations)
{
	dbfield_t *field;
	dbtable_t *table;
	dbtableset_t *tableset;
	dbrelation_t *relation;
	dbtable_t *ptable;
	dbtable_t *rptable;
	dbtable_t *rftable;
	GString *str;
	GString *pstr;
	gint whereind;
	gint andind;
	GSList *list;
	if(!relationset || !primarytable)
		return(NULL);
	/****************************************************************/
	/* Find the primary table					*/
	/****************************************************************/
	tableset=relationset->tableset;
	ptable=dbutil_tableset_get_table(tableset, primarytable);
	if(!ptable)
		return(NULL);
	/****************************************************************/
	/* allocate 'str' to hold the SQL statement			*/
	/****************************************************************/
	str=g_string_sized_new(SQLSTRLEN);
	if(!str)
		return(NULL);
	/****************************************************************/
	/* setup the SELECT fields					*/
	/****************************************************************/
	g_string_append(str, "select");
	field = ptable->fieldset;
	while(field)
		{
		if(field != ptable->fieldset)
			g_string_append(str, ",");
		g_string_sprintfa(str, " %s.%s", ptable->tablename,
				field->dbfield);
		field = field->next;
		}
	/****************************************************************/
	/* setup the FROM tables					*/
	/****************************************************************/
	g_string_sprintfa(str, " from %s", ptable->tablename);
	if(userelations)
		{
		list=relationset->relationlist;
		while(list)
			{
			if( (relation=(dbrelation_t *)list->data) )
				if((dbtable_t *)relation->foreign->table == ptable)
					{
					table=(dbtable_t *)relation->primary->table;
					if(!table_is_cleared(table))
						g_string_sprintfa(str, ", %s",
							table->tablename);
					}
			list=list->next;
			}
		}
	/****************************************************************/
	/* setup the WHERE relations					*/
	/****************************************************************/
	whereind=0;
	andind=0;
	if(userelations)
		{
		list=relationset->relationlist;
		while(list)
			{
			if( (relation=(dbrelation_t *)list->data) )
				{
				if((dbtable_t *)relation->foreign->table == ptable)
					{
					rptable=(dbtable_t *)relation->primary->table;
					rftable=(dbtable_t *)relation->foreign->table;
					if(!table_is_cleared(rptable))
						{
						if(!whereind)
							{
							g_string_append(str, " where");
							whereind++;
							}
						if(andind)
							g_string_append(str, " and");
						g_string_sprintfa(str, " %s.%s = %s.%s",
							rptable->tablename,
							relation->primary->dbfield,
							rftable->tablename,
							relation->foreign->dbfield);
						andind++;
						}
					}
				}
			list=list->next;
			}
		}
	/****************************************************************/
	/* setup the WHERE fields					*/
	/****************************************************************/
	pstr = table_get_where_fields(ptable);
	if(pstr)
		{
		if(!whereind)
			{
			g_string_append(str, " where");
			whereind++;
			}
		if(andind)
			g_string_append(str, " and");
		g_string_append(str, pstr->str);
		g_string_free(pstr, TRUE);
		andind++;
		}

	/****************************************************************/
if(userelations)
	{
	list=relationset->relationlist;
	while(list)
		{
		if( (relation=(dbrelation_t *)list->data) )
			{
			/************************************************************/
			/* Handle relations where the primary table has foreign keys*/
			/* TO the related table(s).									*/
			/************************************************************/
			if((dbtable_t *)relation->foreign->table == ptable)
				{
				rptable=(dbtable_t *)relation->primary->table;
				rftable=(dbtable_t *)relation->foreign->table;
				/********************************************************/
				/* Use Primary key fields...if filled					*/
				/********************************************************/
				field = dbutil_table_get_primary_field(rptable);
				if(field && (field->flags & DBF_HASDATA))
					pstr = field_get_where(field);
				else
					pstr = table_get_where_fields(rptable);
				if(pstr)
					{
					if(!whereind)
						{
						g_string_append(str, " where");
						whereind++;
						}
					if(andind)
						g_string_append(str, " and");
					g_string_append(str, pstr->str);
					g_string_free(pstr, TRUE);
					andind++;
					}
				}
			/************************************************************/
			/* Handle relations where the primary table has foreign keys*/
			/* IN the related table(s).									*/
			/************************************************************/
			if((dbtable_t *)relation->primary->table == ptable)
				{
				rptable=(dbtable_t *)relation->primary->table;
				rftable=(dbtable_t *)relation->foreign->table;
				/********************************************************/
				/* Use Foreign key fields...if filled					*/
				/********************************************************/
				field = relation->foreign;
				pstr = relation_get_foreign_where(relation);
				if(pstr)
					{
					if(!whereind)
						{
						g_string_append(str, " where");
						whereind++;
						}
					if(andind)
						g_string_append(str, " and");
					g_string_append(str, pstr->str);
					g_string_free(pstr, TRUE);
					andind++;
					}
				}
			}
		list=list->next;
		}
	}
return(str);
}


GString *
dbutil_relationset_insert(dbrelationset_t *relationset,
				gchar *primarytable)
	{
	dbtable_t *ptable;
	dbtableset_t *tableset;
	dbfield_t *field;
	GString *str;
	GString *data;
	gint ind;
	if(!relationset || !primarytable)
		return(NULL);
	/****************************************************************/
	/* Find the primary table					*/
	/****************************************************************/
	tableset=relationset->tableset;
	ptable=dbutil_tableset_get_table(tableset, primarytable);
	if(!ptable)
		{
		return(NULL);
		}
	/****************************************************************/
	/* allocate 'str' to hold the SQL statement			*/
	/****************************************************************/
	str=g_string_sized_new(SQLSTRLEN);
	if(!str)
		return(NULL);
	/****************************************************************/
	/* setup the field names					*/
	/****************************************************************/
	g_string_sprintfa(str, "insert into %s (", ptable->tablename);
	ind=0;
	field = ptable->fieldset;
	while(field)
		{
		if(!(field->flags & DBF_PRIMARYKEY))
			{
			if(ind)
				{
				g_string_append(str, ",");
				}
			g_string_sprintfa(str, " %s", field->dbfield);
			ind++;
			}
		field = field->next;
		}
	g_string_append(str, ") values (");
	/****************************************************************/
	/* setup the field values					*/
	/****************************************************************/
	ind=0;
	field = ptable->fieldset;
	while(field)
		{
		if(!(field->flags & DBF_PRIMARYKEY))
			{
			if(ind)
				{
				g_string_append(str, ",");
				}
			data=field_get_sql_value(field);
			if(data->str[0])
				g_string_sprintfa(str, " %s", data->str);
			else
				g_string_append(str, " 0");
			g_string_free(data, TRUE);
			ind++;
			}
		field = field->next;
		}
	g_string_append(str, ")");
	return(str);
	}


GString *
dbutil_relationset_update(dbrelationset_t *relationset,
				gchar *primarytable)
	{
	dbtable_t *ptable;
	dbtableset_t *tableset;
	dbfield_t *field;
	GString *str;
	GString *data;
	gint ind;
	if(!relationset || !primarytable)
		return(NULL);
	/****************************************************************/
	/* Find the primary table					*/
	/****************************************************************/
	tableset=relationset->tableset;
	ptable=dbutil_tableset_get_table(tableset, primarytable);
	if(!ptable)
		{
		return(NULL);
		}
	/****************************************************************/
	/* allocate 'str' to hold the SQL statement			*/
	/****************************************************************/
	str=g_string_sized_new(SQLSTRLEN);
	if(!str)
		return(NULL);
	/****************************************************************/
	/* build the update statement					*/
	/****************************************************************/
	g_string_sprintfa(str, "update %s set", ptable->tablename);
	ind=0;
	field = ptable->fieldset;
	while(field)
		{
		if( (!(field->flags&DBF_PRIMARYKEY)) &&
		    (!(field->flags&DBF_UNIQUEPART)) )
			{
			if(ind)
				{
				g_string_append(str, ",");
				}
			data=field_get_sql_value(field);
			g_string_sprintfa(str, " %s = %s", field->dbfield,
				data->str);
			g_string_free(data, TRUE);
			ind++;
			}
		field = field->next;
		}
	/****************************************************************/
	/* setup the where statement					*/
	/****************************************************************/
	/* if the table has a primary key... use it.			*/
	/****************************************************************/
	ind=0;
	field = dbutil_table_get_primary_field(ptable);
	if(field)
		{
		if(field->flags & DBF_HASDATA)
			{
			data=field_get_sql_value(field);
			g_string_sprintfa(str, " where %s = %s",
				field->dbfield,
				data->str);
			g_string_free(data, TRUE);
			}
		}
	/****************************************************************/
	/* else if the table has fields that make a unique key...	*/
	/* use those fields.						*/
	/****************************************************************/
	else
		{
		ind=0;
		field=ptable->fieldset;
		while(field)
			{
			if((field->flags & DBF_UNIQUEPART) &&
				(field->flags & DBF_HASDATA))
				{
				if(ind)
					g_string_append(str, " and");
				else
					g_string_append(str, " where");
				data=field_get_sql_value(field);
				g_string_sprintfa(str, " %s = %s",
					field->dbfield,
					data->str);
				g_string_free(data, TRUE);
				ind++;
				}
			field=field->next;
			}
		}
	return(str);
	}

GString *
dbutil_relationset_delete(dbrelationset_t *relationset,
				char *primarytable)
	{
	dbtable_t *ptable;
	dbtableset_t *tableset;
	dbfield_t *field;
	GString *str;
	GString *data;
	gint ind;
	if(!relationset || !primarytable)
		return(NULL);
	/****************************************************************/
	/* Find the primary table					*/
	/****************************************************************/
	tableset=relationset->tableset;
	ptable=dbutil_tableset_get_table(tableset, primarytable);
	if(!ptable)
		{
		return(NULL);
		}
	/****************************************************************/
	/* allocate 'str' to hold the SQL statement			*/
	/****************************************************************/
	str=g_string_sized_new(SQLSTRLEN);
	if(!str)
		return(NULL);
	/****************************************************************/
	/* build the delete statement					*/
	/****************************************************************/
	g_string_sprintfa(str, "delete from %s", ptable->tablename);
	/****************************************************************/
	/* setup the where statement					*/
	/****************************************************************/
	/* if the table has a primary key... use it.			*/
	/****************************************************************/
	ind=0;
	field = dbutil_table_get_primary_field(ptable);
	if(field)
		{
		if(field->flags & DBF_HASDATA)
			{
			data=field_get_sql_value(field);
			g_string_sprintfa(str, " where %s = %s",
				field->dbfield,
				data->str);
			g_string_free(data, TRUE);
			}
		}
	/****************************************************************/
	/* else if the table has fields that make a unique key...	*/
	/* use those fields.						*/
	/****************************************************************/
	else
		{
		ind=0;
		field=ptable->fieldset;
		while(field)
			{
			if((field->flags & DBF_UNIQUEPART) &&
				(field->flags & DBF_HASDATA))
				{
				if(ind)
					g_string_append(str, " and");
				else
					g_string_append(str, " where");
				data=field_get_sql_value(field);
				g_string_sprintfa(str, " %s = %s",
					field->dbfield,
					data->str);
				g_string_free(data, TRUE);
				ind++;
				}
			field=field->next;
			}
		}
	return(str);
	}


void
dbutil_relationset_select_free(relselect_t *relselect)
	{
	if(!relselect)
		return;
	if(relselect->sqlstr)
		g_string_free(relselect->sqlstr, TRUE);
	g_free(relselect);
	relselect=NULL;
	return;
	}


/************************************************************************/
/* dbbrowser functions							*/
/************************************************************************/
dbbrowser_t *
dbutil_browser_new(gchar *browsername, dbrelationset_t *dbrelationset,
				gchar *primarytable)
	{
	dbbrowser_t *browser;
	dbtable_t *table;
	if(!browsername || !dbrelationset || !primarytable)
		return(NULL);
	table=dbutil_tableset_get_table(dbrelationset->tableset, primarytable);
	if(!table)
		return(NULL);
	browser=(dbbrowser_t *)g_malloc(sizeof(dbbrowser_t));
	if(!browser)
		return(NULL);
	browser->browsername=g_strdup(browsername);
	browser->relationset=dbrelationset;
	browser->primarytable=table;
	browser->fieldlist = g_slist_alloc();
	browser->flags=0;
	return(browser);
	}

void
dbutil_browser_free(dbbrowser_t *dbbrowser)
	{
	if(!dbbrowser)
		return;
	if(dbbrowser->browsername)
		g_free(dbbrowser->browsername);
	if(dbbrowser->fieldlist)
		{
		g_slist_free(dbbrowser->fieldlist);
		}
	g_free(dbbrowser);
	dbbrowser=NULL;
	return;
	}

int
dbutil_browser_add_field(dbbrowser_t *dbbrowser,
				dbfield_t *dbfield)
	{
	dbtable_t *table;
	dbtableset_t *tableset;
	if(!dbbrowser || !dbfield)
		return(-1);
	tableset=dbbrowser->relationset->tableset;
	table=(dbtable_t *)dbfield->table;
	if(!dbutil_tableset_get_table(tableset, table->tablename))
		return(-2);
	g_slist_append(dbbrowser->fieldlist, (gpointer)dbfield);
	return(0);
	}

gint
dbutil_browser_get_primary_field_offset(dbbrowser_t *dbbrowser)
	{
	GSList *list;
	dbfield_t *field;
	gint count=0;
	if(!dbbrowser)
		return(-1);
	field=dbutil_table_get_primary_field(dbbrowser->primarytable);
	if(!field)
		return(-2);
	list=dbbrowser->fieldlist;
	while(list)
		{
		if(list->data)
			{
			if(list->data == field)
				return(count);
			count++;
			}
		list=list->next;
		}
	return(-3);
	}

GString *
dbutil_browser_select(dbbrowser_t *dbbrowser)
{
	dbfield_t *field;
	dbtable_t *table;
	dbtableset_t *tableset;
	dbrelation_t *relation;
	dbrelationset_t *relationset;
	dbtable_t *ptable;
	dbtable_t *rptable;
	dbtable_t *rftable;
	GSList *list;
	GString *str;
	GString *pstr;
	gint whereind;
	gint count;
	gint andind;
	gint tablecount;
	if(!dbbrowser)
		return(NULL);
	relationset=dbbrowser->relationset;
	ptable=dbbrowser->primarytable;
	/****************************************************************/
	/* allocate 'str' to hold the SQL statement			*/
	/****************************************************************/
	str=g_string_sized_new(SQLSTRLEN);
	if(!str)
		return(NULL);
	/****************************************************************/
	/* setup the SELECT fields					*/
	/****************************************************************/
	count=0;
	g_string_append(str, "select");
	list = dbbrowser->fieldlist;
	while(list)
		{
		field=(dbfield_t *)list->data;
		if(field)
			{
			if(count)
				g_string_append(str, ",");
			table=(dbtable_t *)field->table;
			g_string_sprintfa(str, " %s.%s",
					table->tablename,
					field->dbfield);
			count++;
			}
		list = list->next;
		}
	/****************************************************************/
	/* setup the FROM tables					*/
	/****************************************************************/
	tablecount=0;
	g_string_append(str, " from ");
	tableset=relationset->tableset;
	list=tableset->tablelist;
	while(list)
		{
		if( (table=(dbtable_t *)list->data) )
			{
			if(table_used_in_browser(dbbrowser, table))
				{
				if(tablecount)
					g_string_append(str, ", ");
				g_string_sprintfa(str, " %s",
						 table->tablename);
				tablecount++;
				}
			}
		list=list->next;
		}
	/****************************************************************/
	/* setup the WHERE relations					*/
	/****************************************************************/
	whereind=0;
	andind=0;
	list=relationset->relationlist;
	while(list)
		{
		if( (relation=(dbrelation_t *)list->data) )
			{
			rptable=(dbtable_t *)relation->primary->table;
			rftable=(dbtable_t *)relation->foreign->table;
			if( table_used_in_browser(dbbrowser, rptable) &&
			    table_used_in_browser(dbbrowser, rftable) )
				{
				if(!whereind)
					{
					g_string_append(str, " where");
					whereind++;
					}
				if(andind)
					g_string_append(str, " and");
				g_string_sprintfa(str, " %s.%s = %s.%s",
					rptable->tablename,
					relation->primary->dbfield,
					rftable->tablename,
					relation->foreign->dbfield);
				andind++;
				}
			}
		list=list->next;
		}
	/****************************************************************/
	/* setup the WHERE fields					*/
	/****************************************************************/
	pstr = table_get_where_fields(ptable);
	if(pstr)
		{
		if(!whereind)
			{
			g_string_append(str, " where");
			whereind++;
			}
		if(andind)
			g_string_append(str, " and");
		g_string_append(str, pstr->str);
		g_string_free(pstr, TRUE);
		andind++;
		}
	/****************************************************************/
	list=relationset->relationlist;
	while(list)
		{
		if( (relation=(dbrelation_t *)list->data) )
			{
			/************************************************************/
			/* Handle relations where the primary table has foreign keys*/
			/* TO the related table(s).									*/
			/************************************************************/
			if((dbtable_t *)relation->foreign->table == ptable)
				{
				rptable=(dbtable_t *)relation->primary->table;
				rftable=(dbtable_t *)relation->foreign->table;
				/********************************************************/
				/* Use Primary key fields...if filled					*/
				/********************************************************/
				field = dbutil_table_get_primary_field(rptable);
				if(field && (field->flags & DBF_HASDATA))
					pstr = field_get_where(field);
				else
					pstr = table_get_where_fields(rptable);
				if(pstr)
					{
					if(!whereind)
						{
						g_string_append(str, " where");
						whereind++;
						}
					if(andind)
						g_string_append(str, " and");
					g_string_append(str, pstr->str);
					g_string_free(pstr, TRUE);
					andind++;
					}
				}
			/************************************************************/
			/* Handle relations where the primary table has foreign keys*/
			/* IN the related table(s).									*/
			/************************************************************/
			if((dbtable_t *)relation->primary->table == ptable)
				{
				rptable=(dbtable_t *)relation->primary->table;
				rftable=(dbtable_t *)relation->foreign->table;
				/********************************************************/
				/* Use Foreign key fields...if filled					*/
				/********************************************************/
				field = relation->foreign;
				pstr = relation_get_foreign_where(relation);
				if(pstr)
					{
					if(!whereind)
						{
						g_string_append(str, " where");
						whereind++;
						}
					if(andind)
						g_string_append(str, " and");
					g_string_append(str, pstr->str);
					g_string_free(pstr, TRUE);
					andind++;
					}
				}
			}
		list=list->next;
		}
	return(str);
	}


/************************************************************************/
/* Helper functions							*/
/************************************************************************/
gint table_is_cleared(dbtable_t *dbtable)
	{
	dbfield_t *field;
	field = dbtable->fieldset;
	while(field)
		{
		if(field->flags & DBF_HASDATA)
			return(0);
		field=field->next;
		}
	return(1);
	}


GString *
table_get_where_fields(dbtable_t *table)
	{
	dbfield_t *field;
	GString *str;
	gint andind;
	if(!table)
		return(NULL);
	str = g_string_sized_new(SQLSTRLEN);
	andind = 0;
	if(!table_is_cleared(table))
		{
		field=table->fieldset;
		while(field)
			{
			if(field->flags & DBF_HASDATA)
				{
				if(andind)
					g_string_append(str, " and");
				if(field->flags & DBF_INT)
					{
					g_string_sprintfa(str, " %s.%s = %s",
						table->tablename,
						field->dbfield,
						(gchar *)field->data);
					}
				else
					{
					if(field->flags & DBF_SQLUSELIKE)
						g_string_sprintfa(str, " %s.%s like '%%%s%%'",
							table->tablename,
							field->dbfield,
							(gchar *)field->data);
					else
						g_string_sprintfa(str, " %s.%s = '%s'",
							table->tablename,
							field->dbfield,
							(gchar *)field->data);
					}
					andind++;
				}
			field=field->next;
			}
		return(str);
		}
	else
		return(NULL);
	}


GString *
field_get_where(dbfield_t *field)
	{
	dbtable_t *table;
	GString *str;
	if(!field)
		return(NULL);
	str = g_string_sized_new(SQLSTRLEN);
	if(field->flags & DBF_HASDATA)
		{
		table = (dbtable_t *)field->table;
		if(field->flags & DBF_INT)
			{
			g_string_sprintfa(str, " %s.%s = %s",
				table->tablename,
				field->dbfield,
				(gchar *)field->data);
			}
		else
			{
			if(field->flags & DBF_SQLUSELIKE)
				g_string_sprintfa(str, " %s.%s like '%%%s%%'",
					table->tablename,
					field->dbfield,
					(gchar *)field->data);
			else
				g_string_sprintfa(str, " %s.%s = '%s'",
					table->tablename,
					field->dbfield,
					(gchar *)field->data);
			}
		return(str);
		}
	else
		return(NULL);
	}


GString *
relation_get_foreign_where(dbrelation_t *relation)
	{
	dbtable_t *table;
	GString *str;
	if(!relation)
		return(NULL);
	str = g_string_sized_new(SQLSTRLEN);
	if(relation->foreign->flags & DBF_HASDATA)
		{
		table = (dbtable_t *)relation->primary->table;
		if(relation->foreign->flags & DBF_INT)
			{
			g_string_sprintfa(str, " %s.%s = %s",
				table->tablename,
				relation->primary->dbfield,
				(gchar *)relation->foreign->data);
			}
		else
			{
			if(relation->foreign->flags & DBF_SQLUSELIKE)
				g_string_sprintfa(str, " %s.%s like '%%%s%%'",
					table->tablename,
					relation->primary->dbfield,
					(char *)relation->foreign->data);
			else
				g_string_sprintfa(str, " %s.%s = '%s'",
					table->tablename,
					relation->primary->dbfield,
					(char *)relation->foreign->data);
			}
		return(str);
		}
	else
		return(NULL);
	}


GString *
field_get_sql_value(dbfield_t *field)
	{
	dbtable_t *table;
	GString *str;
	if(!field)
		return(NULL);
	str = g_string_new(NULL);
	if(!str)
		return(NULL);
	table = (dbtable_t *)field->table;
	if(field->flags & DBF_CHAR)
		g_string_append(str, "'");
	if(field->data)
		g_string_append(str, (gchar *)field->data);
	else
		if(field->flags & DBF_INT)
			g_string_append(str, "0");
	if(field->flags & DBF_CHAR)
		g_string_append(str, "'");
	return(str);
	}

sqlhandle_t *
sqlhandle_new(void)
	{
	sqlhandle_t *handle;
	handle=(sqlhandle_t *)g_malloc(sizeof(sqlhandle_t));
	if(!handle)
		return(NULL);
	handle->relationset=NULL;
	handle->TableListIndex=0;
	handle->primarytable=NULL;
	handle->sqlstr=NULL;
	handle->flags=0;
	handle->multirownumber=0;
	return(handle);
	}

gint
table_used_in_browser(dbbrowser_t *dbbrowser, dbtable_t *dbtable)
	{
	GSList *list;
	dbfield_t *field;
	list=dbbrowser->fieldlist;
	while(list)
		{
		field=(dbfield_t *)list->data;
		if(field)
			if((gpointer)field->table == (gpointer)dbtable)
				return(1);
		list=list->next;
		}
	return(0);
	}


