/************************************************************************/
/* dbutil.h - 								*/
/************************************************************************/
/* dbutil
 * Copyright (C) 1999-2001  Todd Shadburn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/* (04-24-2001) Convert to using GString for the SQL statements.Todd.	*/
/* (12-14-1999) Original creation date.Todd Shadburn			*/
/************************************************************************/

#ifndef _DBUTIL_H_
#define _DBUTIL_H_

#include <glib.h>

/************************************************************************/
/* Predeclare the dbtable_t structure for use in the dbfield_t struct.	*/
/************************************************************************/
typedef struct dbtable_s dbtable_t;

/************************************************************************/
/* dbfield								*/
/************************************************************************/
#define	DBF_INT		0x0001
#define DBF_CHAR	0x0002
#define DBF_OPTIONLIST	0x0004
#define DBF_HASDATA	0x0010
#define DBF_REQUIRED	0x0020
#define DBF_PRIMARYKEY	0x0040
#define DBF_UNIQUEPART	0x0080
#define DBF_SQLUSELIKE	0x0100
#define DBF_NOGUIWIDGET	0x0200

typedef struct dbfield_s
	{
	gchar *dbfield;
	gpointer widget;
	dbtable_t *table;
	gpointer data;
	gint flags;
	struct dbfield_s *next;
	} dbfield_t;

/************************************************************************/
/* dbtable								*/
/************************************************************************/
#define DBT_MULTIROW	0x0100

struct dbtable_s
	{
	gchar *tablename;
	dbfield_t *fieldset;
	gpointer data;
	gpointer widget;
	gint flags;
	};

/************************************************************************/
/* dbtableset								*/
/************************************************************************/
typedef struct dbtableset_s
	{
	gchar *tablesetname;
	GSList *tablelist;
	gpointer data;
	gint flags;
	} dbtableset_t;

/************************************************************************/
/* dbrelation								*/
/************************************************************************/
typedef struct dbrelation_s
	{
	gchar *relationname;
	dbfield_t *primary;
	dbfield_t *foreign;
	gpointer data;
	gint flags;
	} dbrelation_t;

/************************************************************************/
/* dbrelationset							*/
/************************************************************************/
typedef struct dbrelationset_s
	{
	gchar *relationsetname;
	dbtableset_t *tableset;
	GSList *relationlist;
	gpointer data;
	gint flags;
	} dbrelationset_t;

/************************************************************************/
/* dbbrowser								*/
/************************************************************************/
typedef struct dbbrowser_s
	{
	gchar *browsername;
	dbrelationset_t *relationset;
	dbtable_t *primarytable;
	GSList *fieldlist;
	gint flags;	
	} dbbrowser_t;

/************************************************************************/
/* relselect_t								*/
/************************************************************************/
#define SQLH_SELECT	0x1000
#define SQLH_INSERT	0x2000
#define SQLH_UPDATE	0x4000
#define SQLH_DELETE	0x8000

typedef struct
	{
	dbrelationset_t *relationset;
	gint TableListIndex;
	dbtable_t *primarytable;
	GString *sqlstr;
	gint multirownumber;
	gint flags;
	} relselect_t;

typedef relselect_t sqlhandle_t;

/************************************************************************/
/* public function prototypes						*/
/************************************************************************/
dbtable_t *
dbutil_table_new(gchar *tablename);

void
dbutil_table_free(dbtable_t *dbtable);

gint
dbutil_table_add_field(dbtable_t *dbtable, dbfield_t *dbfield);

dbfield_t *
dbutil_table_get_primary_field(dbtable_t *dbtable);

dbfield_t *
dbutil_table_get_field(dbtable_t *dbtable, gchar *fieldname);

gint
dbutil_table_required_fields_filled(dbtable_t * dbtable);

void
dbutil_table_set_flags(dbtable_t *dbtable, gint flags);

void
dbutil_table_clear_flags(dbtable_t *dbtable, gint flags);

void
dbutil_table_set_widget(dbtable_t *dbtable, gpointer widget);

/************************************************************************/
dbtableset_t *
dbutil_tableset_new(gchar *tablesetname);

void
dbutil_tableset_free(dbtableset_t *dbtableset);

gint
dbutil_tableset_add_table(dbtableset_t *dbtableset, dbtable_t *dbtable);

dbtable_t *
dbutil_tableset_get_table(dbtableset_t *dbtableset, gchar *tablename);

/************************************************************************/
dbfield_t *
dbutil_field_new(void *widget, gchar *field);

void
dbutil_field_free(dbfield_t *dbfield);

void
dbutil_field_set_data(dbfield_t *dbfield, gpointer data);

void
dbutil_field_clear_data(dbfield_t *dbfield);

void
dbutil_field_set_primary(dbfield_t *dbfield);

void
dbutil_field_set_flags(dbfield_t *dbfield, gint flags);

void
dbutil_field_clear_flags(dbfield_t *dbfield, gint flags);

/************************************************************************/
dbrelation_t *
dbutil_relation_new(char *name, dbfield_t *primary, dbfield_t *foreign);

void
dbutil_relation_free(dbrelation_t *dbrelation);

void
dbutil_relation_set_data(dbrelation_t *dbrelation, gpointer data);

/************************************************************************/
dbrelationset_t *
dbutil_relationset_new(gchar *relationsetname, dbtableset_t *tableset);

void
dbutil_relationset_free(dbrelationset_t *dbrelationset);

void
dbutil_relationset_full_free(dbrelationset_t *dbrelationset);

gint
dbutil_relationset_add_relation(dbrelationset_t *dbrelationset,
				dbrelation_t *dbrelation);

GString *
dbutil_relationset_select(dbrelationset_t *relationset,
				gchar *primarytable, gint userelations);

GString *
dbutil_relationset_insert(dbrelationset_t *relationset,
				gchar *primarytable);

GString *
dbutil_relationset_update(dbrelationset_t *relationset,
				gchar *primarytable);

GString *
dbutil_relationset_delete(dbrelationset_t *relationset,
				gchar *primarytable);

void
dbutil_relationset_select_free(relselect_t *relselect);

/************************************************************************/
dbbrowser_t *
dbutil_browser_new(gchar *browsername, dbrelationset_t *dbrelationset,
				gchar *primarytable);

void
dbutil_browser_free(dbbrowser_t *dbbrowser);

gint
dbutil_browser_add_field(dbbrowser_t *dbbrowser,
				dbfield_t *dbfield);

gint
dbutil_browser_get_primary_field_offset(dbbrowser_t *dbbrowser);

GString *
dbutil_browser_select(dbbrowser_t *dbbrowser);

#endif

