/************************************************************************/
/* gtkdbutil.c - 							*/
/************************************************************************/
/* dbutil
 * Copyright (C) 1999-2001  Todd Shadburn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/* (04-24-2001) Convert to using GString for the SQL statements.Todd.	*/
/* (04-14-2001) Taken from dbutil.c...The GTK stuff. Todd		*/
/* (01-05-2000) Added str variable to dbutil_table_sync_with_gui because*/
/*		gtk_entry_get_text() returns a empty string("") instead	*/
/*		of a NULL when a GtkEntry is blank. Todd		*/
/* (01-04-2000) Some cleanup, added table_get_where_fields() and moved	*/
/*		the GTK functions back into the API. Todd.		*/
/* (01-03-2000) Working version of fbutil_relationset_select_new().Todd	*/
/* (12-14-1999) Original creation date.Todd Shadburn			*/
/************************************************************************/

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdlib.h>
#include <glib.h>
#include <gtk/gtk.h>

#ifdef HAVE_LIBGTKEXTRA
#include <gtkextra/gtksheet.h>
#endif

#include "gtkdbutil.h"

#define SQLSTRLEN	256

/************************************************************************/
/* Forward declarations							*/
/************************************************************************/
gint table_is_cleared(dbtable_t *dbtable);
gchar * table_get_where_fields(dbtable_t *table);
gchar * field_get_where(dbfield_t *field);
gchar * relation_get_foreign_where(dbrelation_t *relation);
gchar * field_get_sql_value(dbfield_t *field);
sqlhandle_t *sqlhandle_new(void);
gint table_used_in_browser(dbbrowser_t *dbbrowser, dbtable_t *dbtable);
gint gtk_option_menu_get_active(GtkOptionMenu *option_menu);
gint gtk_option_menu_set_active(GtkOptionMenu *option_menu, gint index);
gchar *gtk_option_menu_get_active_text(GtkOptionMenu *option_menu);

/************************************************************************/
/* public function prototypes						*/
/************************************************************************/
void		dbutil_gtk_table_sync_with_gui(dbtable_t *dbtable)
	{
	dbfield_t *field;
	GString *svalue=NULL;
	gchar *str=NULL;
	gint value;
	if(!dbtable)
		return;
	svalue=g_string_new(NULL);
	field = dbtable->fieldset;
	while(field)
		{
		if(field->widget)
			{
			if(GTK_IS_ENTRY(field->widget))
				g_string_assign(svalue, gtk_entry_get_text(GTK_ENTRY(field->widget)));
			if(GTK_IS_OPTION_MENU(field->widget))
				{
				if((field->flags & DBF_INT))
					{
					value=gtk_option_menu_get_active(GTK_OPTION_MENU(field->widget));
					if(value)
						g_string_sprintf(svalue, "%d", value);
					else
						g_string_truncate(svalue, 0);
					}
				else
					{
					str=gtk_option_menu_get_active_text(GTK_OPTION_MENU(field->widget));
					if(!str)
						{
						g_string_truncate(svalue, 0);
						}
					}
				}
			if(GTK_IS_TOGGLE_BUTTON(field->widget))
				{
				value=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(field->widget));
				if(value)
					g_string_sprintf(svalue, "%d", value);
				else
					g_string_truncate(svalue, 0);
				}
			if(svalue->str[0] != 0x00)
				dbutil_field_set_data(field, (void *)svalue->str);
			else
				dbutil_field_clear_data(field);
			}
		field = field->next;
		}
	g_string_free(svalue, TRUE);
	return;
	}


void		dbutil_gtk_field_clear(dbfield_t *dbfield)
	{
	if(!dbfield)
		return;
	dbutil_field_clear_data(dbfield);
	if(dbfield->widget)
		{
		if(GTK_IS_ENTRY(dbfield->widget))
			gtk_entry_set_text(GTK_ENTRY(dbfield->widget), "");
		if(GTK_IS_OPTION_MENU(dbfield->widget))
			gtk_option_menu_set_active(
					GTK_OPTION_MENU(dbfield->widget),
					0);
		if(GTK_IS_TOGGLE_BUTTON(dbfield->widget))
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dbfield->widget),
							FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(dbfield->widget),
					 TRUE);
		}
	return;
	}


void		dbutil_gtk_field_fill(dbfield_t *dbfield, gchar *data)
	{
	if(!dbfield)
		return;
	dbutil_field_set_data(dbfield, data);
	if(dbfield->widget)
		{
		if(GTK_IS_ENTRY(dbfield->widget))
			gtk_entry_set_text(GTK_ENTRY(dbfield->widget), data);
		if(GTK_IS_OPTION_MENU(dbfield->widget))
			gtk_option_menu_set_active(dbfield->widget, atoi(data));
		if( (dbfield->flags & DBF_PRIMARYKEY) ||
		    (dbfield->flags & DBF_UNIQUEPART) )
			gtk_widget_set_sensitive(GTK_WIDGET(dbfield->widget),
						 FALSE);
		if(GTK_IS_TOGGLE_BUTTON(dbfield->widget))
			gtk_toggle_button_set_active(dbfield->widget, atoi(data));
		}
	return;
	}


void		dbutil_gtk_table_clear_fields(dbtable_t *dbtable)
	{
	dbfield_t *field;
	if(!dbtable)
		return;
	field = dbtable->fieldset;
	while(field)
		{
		dbutil_field_clear_data(field);
		dbutil_gtk_field_clear(field);
		field = field->next;
		}
	return;
	}


void		dbutil_gtk_table_fill_fields(dbtable_t *dbtable, gchar **data)
	{
	dbfield_t *field;
	gint count;
	if(!dbtable || !data)
		return;
	field = dbtable->fieldset;
	count=0;
	while(field && data[count])
		{
		dbutil_field_set_data(field, data[count]);
		dbutil_gtk_field_fill(field, data[count]);
		field = field->next;
		count++;
		}
	return;
	}

void
dbutil_gtk_table_set_fields_sensitive(dbtable_t *dbtable, gboolean sensitive)
	{
	dbfield_t *field;
	if(!dbtable)
		return;
	field = dbtable->fieldset;
	while(field)
		{
		if(field->widget)
			{
			if( (!(field->flags&DBF_PRIMARYKEY)) &&
			    (!(field->flags&DBF_UNIQUEPART)) )
				{
				gtk_widget_set_sensitive(GTK_WIDGET(field->widget),
							 sensitive);
				}
			}
		field = field->next;
		}
	return;
	}

void
dbutil_gtk_relationset_sync_with_gui(dbrelationset_t *dbrelationset)
	{
	dbtable_t *table;
	dbrelation_t *relation;
	GSList *list;
	if(!dbrelationset)
		return;
	list=dbrelationset->tableset->tablelist;
	while(list)
		{
		if( (table=(dbtable_t *)list->data) )
			dbutil_gtk_table_sync_with_gui(table);
		list=list->next;
		}

	list=dbrelationset->relationlist;
	while(list)
		{
		if( (relation=(dbrelation_t *)list->data) )
			{
			if(relation->primary->flags & DBF_HASDATA)
				{
				dbutil_field_clear_data(relation->foreign);
				dbutil_field_set_data(relation->foreign,
							relation->primary->data);
				}
			}
		list=list->next;
		}
	return;
	}


sqlhandle_t *
dbutil_gtk_relationset_multirow_start(dbrelationset_t *relationset,
				gchar *primarytable,
				gint statementtype)
	{
	dbtable_t *table;
	sqlhandle_t *handle;
	if(!relationset || !primarytable || !statementtype)
		return(NULL);
	table=dbutil_tableset_get_table(relationset->tableset, primarytable);
	if(!table)
		return(NULL);
	if(!table->flags & DBT_MULTIROW)
		return(NULL);
	if(statementtype == SQLH_SELECT)
		return(NULL);
	handle=sqlhandle_new();
	if(!handle)
		return(NULL);
	handle->flags=statementtype;
	handle->relationset=relationset;	
	handle->primarytable=table;
	handle->multirownumber=-1;
	if(dbutil_gtk_relationset_multirow_next(handle))
		{
		g_free(handle);
		return(NULL);
		}
	return(handle);
	}


int
dbutil_gtk_relationset_multirow_next(sqlhandle_t *handle)
	{
#ifdef HAVE_LIBGTKEXTRA
	GtkSheet *widget = NULL;
#else
	GtkWidget *widget = NULL;
#endif
	dbrelationset_t *relationset;
	dbtable_t *table;
	dbfield_t *field;
	gint column;
	gint row;
	gint validdata;
	gchar *data = NULL;
	if(!handle || !handle->primarytable)
		return(-1);
	relationset=handle->relationset;
	table=handle->primarytable;
	field=table->fieldset;
#ifdef HAVE_LIBGTKEXTRA
	widget=(GtkSheet *)table->widget;
#endif
	if(!widget)
		return(-1);
	column=0;
	validdata=0;
	handle->multirownumber++;
	row=handle->multirownumber;
#ifdef HAVE_LIBGTKEXTRA
	if(row > widget->maxrow)
		return(-2);
#endif
	/****************************************************************/
	/* sync the dbutil structures based on the table-widget row data*/
	/****************************************************************/
	while(field)
		{
		if(!(field->flags & DBF_NOGUIWIDGET))
			{
#ifdef HAVE_LIBGTKADDONS
			data=gtk_sheet_cell_get_text(widget, row, column);
#endif
			if(data)
				{
				dbutil_field_set_data(field, data);
				validdata++;
				}
			else
				dbutil_field_clear_data(field);
			column++;
			}
		field=field->next;
		}
	if(!validdata)
		return(-3);
	if(handle->sqlstr)
		g_string_free(handle->sqlstr, TRUE);
	switch(handle->flags)
		{
		case SQLH_INSERT:
			handle->sqlstr=dbutil_relationset_insert(relationset,
								table->tablename);
			break;
		case SQLH_UPDATE:
			handle->sqlstr=dbutil_relationset_update(relationset,
								table->tablename);
			break;
		case SQLH_DELETE:
			handle->sqlstr=dbutil_relationset_delete(relationset,
								table->tablename);
			break;
		}
	if(!handle->sqlstr)
		{
		return(-4);
		}
	return(0);
	}


void
dbutil_gtk_relationset_multirow_free(sqlhandle_t *handle)
	{
	if(handle)
		{
		if(handle->sqlstr)
			g_string_free(handle->sqlstr, TRUE);
		g_free(handle);
		handle=NULL;
		}
	return;
	}

gint
gtk_option_menu_get_active(GtkOptionMenu *option_menu)
{
GtkWidget *menu;
GList *list;
GtkWidget *widget;
if(!option_menu)
	return(-1);
menu=gtk_option_menu_get_menu(option_menu);
list=GTK_MENU_SHELL(menu)->children;
widget=gtk_menu_get_active(GTK_MENU(menu));
return(g_list_index(list, widget));
}

gint
gtk_option_menu_set_active(GtkOptionMenu *option_menu, gint index)
{
if(!option_menu)
	return(-1);
gtk_option_menu_set_history(option_menu, index);
return(0);
}

gchar *
gtk_option_menu_get_active_text(GtkOptionMenu *option_menu)
{
GtkLabel label;
GtkWidget *child;
gchar *text; 
if(!option_menu)
	return(NULL);
if (GTK_BIN (option_menu)->child)
	{ 
	child = GTK_BIN (option_menu)->child; 
	/* do stuff with child */ 
	if (GTK_IS_ACCEL_LABEL (child)) 
		{
		label=GTK_ACCEL_LABEL(child)->label;
			gtk_label_get (GTK_LABEL (&label), &text); 
		}
	}
return(text);
}


