/************************************************************************/
/* basictest.c - Test the dbutil library's basic functions.		*/
/************************************************************************/
/* dbutil
 * Copyright (C) 1999-2001  Todd Shadburn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/* (04-25-2001) v0.1 Original creation date. Todd Shadburn		*/
/************************************************************************/

#include <stdio.h>
#include <dbutil.h>

int main(void)
	{
	gint rc;
	GString *str;
	dbtable_t *t1;
	dbtable_t *t2;
	dbfield_t *f11;
	dbfield_t *f12;
	dbfield_t *f21;
	dbfield_t *f22;
	dbtableset_t *ts;
	dbrelation_t *r;
	dbrelationset_t *rs;
	dbbrowser_t *b;

	/* Create table1 and add two fields	*/
	printf("Creating table1................");
	t1 = dbutil_table_new("t1");
	if(!t1)
		{
		printf("Failed to alloc table1\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Creating field11...............");
	f11 = dbutil_field_new(NULL, "f11");
	if(!f11)
		{
		printf("Failed to alloc field11\n");
		return(-1);
		}
	dbutil_field_set_primary(f11);
	printf("Ok!\n");

	printf("Creating field12...............");
	f12 = dbutil_field_new(NULL, "f12");
	if(!f12)
		{
		printf("Failed to alloc field12\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding field11 to table1.......");
	rc = dbutil_table_add_field(t1, f11);
	if(rc)
		{
		printf("Failed to add field to table1\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding field12 to table1.......");
	rc = dbutil_table_add_field(t1, f12);
	if(rc)
		{
		printf("Failed to add field to table1\n");
		return(-1);
		}
	printf("Ok!\n");

	/* Create table2 and add two fields	*/
	printf("Creating table2................");
	t2 = dbutil_table_new("t2");
	if(!t2)
		{
		printf("Failed to alloc table2\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Creating field21...............");
	f21 = dbutil_field_new(NULL, "f21");
	if(!f21)
		{
		printf("Failed to alloc field21\n");
		return(-1);
		}
	dbutil_field_set_primary(f21);
	printf("Ok!\n");

	printf("Creating field22...............");
	f22 = dbutil_field_new(NULL, "f22");
	if(!f22)
		{
		printf("Failed to alloc field22\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding field21 to table2.......");
	rc = dbutil_table_add_field(t2, f21);
	if(rc)
		{
		printf("Failed to add field to table1\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding field22 to table2.......");
	rc = dbutil_table_add_field(t2, f22);
	if(rc)
		{
		printf("Failed to add field to table1\n");
		return(-1);
		}
	printf("Ok!\n");

	/* Create a tableset and add the two tables	*/
	printf("Creating tableset..............");
	ts = dbutil_tableset_new("ts");
	if(!ts)
		{
		printf("Failed to alloc tableset\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding table1 to tableset......");
	rc = dbutil_tableset_add_table(ts, t1);
	if(rc)
		{
		printf("Failed to add table1 to tableset\n");
		return(-1);
		}
	printf("Ok!\n");

	printf("Adding table2 to tableset......");
	rc = dbutil_tableset_add_table(ts, t2);
	if(rc)
		{
		printf("Failed to add table2 to tableset\n");
		return(-1);
		}
	printf("Ok!\n");

	/* Create a relation between two of the fields 	*/
	printf("Creating relation..............");
	r = dbutil_relation_new("r", f21, f11);
	if(!r)
		{
		printf("Failed to alloc relation\n");
		return(-1);
		}
	printf("Ok!\n");

	/* Create a relationset to bring it all together	*/
	printf("Creating relationset...........");
	rs = dbutil_relationset_new("rs", ts);
	if(!rs)
		{
		printf("Failed to alloc relationset\n");
		return(-1);
		}
	printf("Ok!\n");

	/* Add the relationship to the relationset	*/
	printf("Adding relation to relationset.");
	rc = dbutil_relationset_add_relation(rs, r);
	if(rc)
		{
		printf("Failed to add relation to relationset\n");
		return(-1);
		}
	printf("Ok!\n\n");

	/* Perform a SELECT on table1 only(local, no relations used)	*/
	str = dbutil_relationset_select(rs, "t1", 0);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("Local open SELECT on table1:\n%s\n\n", str->str);
	if(g_strcasecmp(str->str, "select t1.f11, t1.f12 from t1"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	g_string_free(str, TRUE);

	/* Perform a SELECT on table2 only(local, no relations used)	*/
	str = dbutil_relationset_select(rs, "t2", 0);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("Local open SELECT on table2:\n%s\n\n", str->str);
	if(g_strcasecmp(str->str, "select t2.f21, t2.f22 from t2"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	g_string_free(str, TRUE);

	/* Perform a SELECT on table1 with a field value set	*/
	/* which will NOT cause the relationship to be used	*/
	dbutil_field_set_data(f11, "test");
	str = dbutil_relationset_select(rs, "t1", 1);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("SELECT on table1 with a field value set:\n%s\n\n", str->str);
	if(g_strcasecmp(str->str,
		"select t1.f11, t1.f12 from t1 where t1.f11 = 'test'"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	dbutil_field_clear_data(f11);
	g_string_free(str, TRUE);

	/* Perform a SELECT on table1 with two field values set	*/
	/* which will NOT cause the relationship to be used	*/
	dbutil_field_set_data(f11, "test");
	dbutil_field_set_data(f12, "record");
	str = dbutil_relationset_select(rs, "t1", 1);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("SELECT on table1 with two field values set:\n%s\n\n", str->str);
	if(g_strcasecmp(str->str,
		"select t1.f11, t1.f12 from t1 where t1.f11 = 'test' and t1.f12 = 'record'"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	dbutil_field_clear_data(f11);
	dbutil_field_clear_data(f12);
	g_string_free(str, TRUE);

	/* Perform a SELECT on table1 with a field value set in table2	*/
	/* which will cause the relationship to be used			*/
	dbutil_field_set_data(f21, "test");
	str = dbutil_relationset_select(rs, "t1", 1);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("SELECT on table1 with a field value set in table2:\n%s\n\n",
		str->str);
	if(g_strcasecmp(str->str,
		"select t1.f11, t1.f12 from t1, t2 where t2.f21 = t1.f11 and t2.f21 = 'test'"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	dbutil_field_clear_data(f21);
	g_string_free(str, TRUE);

	/* Perform a browser SELECT using fields from both t1 and t2	*/
	/* which will cause the relationship to be used			*/
	b = dbutil_browser_new("b", rs, "t1");
	if(!b)
		{
		printf("Failed to alloc browser\n");
		return(-1);
		}
	rc = dbutil_browser_add_field(b, f11);
	if(rc)
		{
		printf("Failed to add field to browser\n");
		return(-1);
		}
	rc = dbutil_browser_add_field(b, f12);
	if(rc)
		{
		printf("Failed to add field to browser\n");
		return(-1);
		}
	rc = dbutil_browser_add_field(b, f21);
	if(rc)
		{
		printf("Failed to add field to browser\n");
		return(-1);
		}
	rc = dbutil_browser_add_field(b, f22);
	if(rc)
		{
		printf("Failed to add field to browser\n");
		return(-1);
		}
	dbutil_field_set_data(f21, "test");
	str = dbutil_browser_select(b);
	if(!str)
		{
		printf("Failed to alloc SQL statement\n");
		return(-1);
		}
	printf("Browser SELECT on table1 and table2 with a field value set in table2:\n%s\n\n",
		str->str);
	if(g_strcasecmp(str->str,
		"select t1.f11, t1.f12, t2.f21, t2.f22 from  t1,  t2 where t2.f21 = t1.f11 and t2.f21 = 'test'"))
		{
		printf("SQL statement built incorrectly\n");
		return(-1);
		}
	g_string_free(str, TRUE);

	/* Free the relationset and all objects which associated	*/
	/* with it(relations, tables, tablesets, fields).		*/
	printf("Freeing all structures in one swoop\n");
	dbutil_relationset_full_free(rs);

	/* That's all!!!	*/
	return(0);
	}


