/************************************************************************/
/* gtkexample.c - GTK+/dbutil example application.			*/
/************************************************************************/
/* dbutil
 * Copyright (C) 1999-2001  Todd Shadburn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/************************************************************************/
/* History:								*/
/* (04-30-2001) v0.1 Original creation date. Todd Shadburn		*/
/************************************************************************/

#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkdbutil.h>

#define TABLE_CONTACT	"contact"

/************************************************************************/
/* Global Variables							*/
/************************************************************************/
GtkWidget *wMain;
dbtable_t *tcontact;
dbfield_t *fcontactid;
dbfield_t *fcontactname;
dbfield_t *fcontactaddr1;
dbfield_t *fcontactaddr2;
dbfield_t *fcontactcity;
dbfield_t *fcontactstate;
dbfield_t *fcontactzip;
dbfield_t *fcontactphone;
dbfield_t *fcontactfax;
dbfield_t *fcontactemail;
dbtableset_t *tscontact;
dbrelationset_t *rscontact;

/************************************************************************/
/* Callback Function Prototypes						*/
/************************************************************************/
GtkWidget* create_wMain(void);
gboolean on_wMain_delete_event(GtkWidget *widget,
                               GdkEvent *event,
                               gpointer user_data);
void on_buttonContactSearch_pressed(GtkButton *button, gpointer user_data);
void on_buttonContactNew_pressed(GtkButton *button, gpointer user_data);
void on_buttonContactUpdate_pressed(GtkButton *button, gpointer user_data);
void on_buttonContactDelete_pressed(GtkButton *button, gpointer user_data);
void on_buttonClear_pressed(GtkButton *button, gpointer user_data);
void on_buttonQuit_pressed(GtkButton *button, gpointer user_data);


/************************************************************************/
/* main()								*/
/************************************************************************/
int main (int argc, char *argv[])
{
gtk_set_locale ();
gtk_init (&argc, &argv);
wMain = create_wMain ();
gtk_widget_show (wMain);
gtk_main ();
return 0;
}


/************************************************************************/
/* Callback Function Declarations					*/
/************************************************************************/
GtkWidget*
create_wMain ()
{
GtkWidget *wMain;
GtkWidget *vboxMain;
GtkWidget *hboxContact;
GtkWidget *vboxContactLabels;
GtkWidget *label1;
GtkWidget *label2;
GtkWidget *label3;
GtkWidget *label4;
GtkWidget *label5;
GtkWidget *label6;
GtkWidget *label7;
GtkWidget *label8;
GtkWidget *vboxContactFields;
GtkWidget *hboxContactId;
GtkWidget *entryContactId;
GtkWidget *entryContactName;
GtkWidget *entryContactAddr1;
GtkWidget *entryContactAddr2;
GtkWidget *hboxContactCSZ;
GtkWidget *entryContactCity;
GtkWidget *label9;
GtkWidget *entryContactState;
GtkWidget *entryContactZip;
GtkWidget *hboxContactPhone;
GtkWidget *entryContactPhone;
GtkWidget *hboxContactFax;
GtkWidget *entryContactFax;
GtkWidget *entryContactEmail;
GtkWidget *vbuttonboxContact;
GtkWidget *buttonContactSearch;
GtkWidget *buttonContactNew;
GtkWidget *buttonContactUpdate;
GtkWidget *buttonContactDelete;
GtkWidget *hbuttonboxMain;
GtkWidget *buttonClear;
GtkWidget *buttonQuit;

wMain = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_object_set_data (GTK_OBJECT (wMain), "wMain", wMain);
gtk_window_set_title (GTK_WINDOW (wMain), "Contacts");

vboxMain = gtk_vbox_new (FALSE, 0);
gtk_widget_ref (vboxMain);
gtk_object_set_data_full (GTK_OBJECT (wMain), "vboxMain", vboxMain,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (vboxMain);
gtk_container_add (GTK_CONTAINER (wMain), vboxMain);

hboxContact = gtk_hbox_new (FALSE, 0);
gtk_widget_ref (hboxContact);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hboxContact", hboxContact,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hboxContact);
gtk_box_pack_start (GTK_BOX (vboxMain), hboxContact, TRUE, TRUE, 0);

vboxContactLabels = gtk_vbox_new (TRUE, 5);
gtk_widget_ref (vboxContactLabels);
gtk_object_set_data_full (GTK_OBJECT (wMain), "vboxContactLabels", vboxContactLabels,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (vboxContactLabels);
gtk_box_pack_start (GTK_BOX (hboxContact), vboxContactLabels, TRUE, TRUE, 0);

label1 = gtk_label_new ("ID:");
gtk_widget_ref (label1);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label1);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label1, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label1), 1, 0.5);

label2 = gtk_label_new ("Name:");
gtk_widget_ref (label2);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label2);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label2, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);

label3 = gtk_label_new ("Address:");
gtk_widget_ref (label3);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label3);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label3, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);

label4 = gtk_label_new ("Address2:");
gtk_widget_ref (label4);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label4);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label4, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label4), 1, 0.5);

label5 = gtk_label_new ("City,State,Zip:");
gtk_widget_ref (label5);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label5);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label5, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label5), 1, 0.5);

label6 = gtk_label_new ("Phone#:");
gtk_widget_ref (label6);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label6);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label6, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label6), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label6), 1, 0.5);

label7 = gtk_label_new ("FAX#:");
gtk_widget_ref (label7);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label7);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label7, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label7), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label7), 1, 0.5);

label8 = gtk_label_new ("Email:");
gtk_widget_ref (label8);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label8);
gtk_box_pack_start (GTK_BOX (vboxContactLabels), label8, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label8), GTK_JUSTIFY_RIGHT);
gtk_misc_set_alignment (GTK_MISC (label8), 1, 0.5);

vboxContactFields = gtk_vbox_new (TRUE, 5);
gtk_widget_ref (vboxContactFields);
gtk_object_set_data_full (GTK_OBJECT (wMain), "vboxContactFields", vboxContactFields,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (vboxContactFields);
gtk_box_pack_start (GTK_BOX (hboxContact), vboxContactFields, TRUE, TRUE, 0);

hboxContactId = gtk_hbox_new (FALSE, 0);
gtk_widget_ref (hboxContactId);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hboxContactId", hboxContactId,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hboxContactId);
gtk_box_pack_start (GTK_BOX (vboxContactFields), hboxContactId, FALSE, FALSE, 0);

entryContactId = gtk_entry_new ();
gtk_widget_ref (entryContactId);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactId", entryContactId,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactId);
gtk_box_pack_start (GTK_BOX (hboxContactId), entryContactId, TRUE, TRUE, 0);
gtk_widget_set_usize (entryContactId, 50, -2);

entryContactName = gtk_entry_new ();
gtk_widget_ref (entryContactName);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactName", entryContactName,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactName);
gtk_box_pack_start (GTK_BOX (vboxContactFields), entryContactName, FALSE, FALSE, 0);

entryContactAddr1 = gtk_entry_new ();
gtk_widget_ref (entryContactAddr1);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactAddr1", entryContactAddr1,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactAddr1);
gtk_box_pack_start (GTK_BOX (vboxContactFields), entryContactAddr1, FALSE, FALSE, 0);

entryContactAddr2 = gtk_entry_new ();
gtk_widget_ref (entryContactAddr2);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactAddr2", entryContactAddr2,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactAddr2);
gtk_box_pack_start (GTK_BOX (vboxContactFields), entryContactAddr2, FALSE, FALSE, 0);

hboxContactCSZ = gtk_hbox_new (FALSE, 2);
gtk_widget_ref (hboxContactCSZ);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hboxContactCSZ", hboxContactCSZ,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hboxContactCSZ);
gtk_box_pack_start (GTK_BOX (vboxContactFields), hboxContactCSZ, TRUE, TRUE, 0);

entryContactCity = gtk_entry_new ();
gtk_widget_ref (entryContactCity);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactCity", entryContactCity,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactCity);
gtk_box_pack_start (GTK_BOX (hboxContactCSZ), entryContactCity, TRUE, TRUE, 0);

label9 = gtk_label_new (", ");
gtk_widget_ref (label9);
gtk_object_set_data_full (GTK_OBJECT (wMain), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (label9);
gtk_box_pack_start (GTK_BOX (hboxContactCSZ), label9, FALSE, FALSE, 0);
gtk_label_set_justify (GTK_LABEL (label9), GTK_JUSTIFY_LEFT);
gtk_misc_set_alignment (GTK_MISC (label9), 1, 0.5);

entryContactState = gtk_entry_new_with_max_length (2);
gtk_widget_ref (entryContactState);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactState", entryContactState,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactState);
gtk_box_pack_start (GTK_BOX (hboxContactCSZ), entryContactState, FALSE, TRUE, 0);
gtk_widget_set_usize (entryContactState, 30, -2);

entryContactZip = gtk_entry_new_with_max_length (10);
gtk_widget_ref (entryContactZip);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactZip", entryContactZip,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactZip);
gtk_box_pack_start (GTK_BOX (hboxContactCSZ), entryContactZip, FALSE, TRUE, 0);
gtk_widget_set_usize (entryContactZip, 85, -2);

hboxContactPhone = gtk_hbox_new (FALSE, 0);
gtk_widget_ref (hboxContactPhone);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hboxContactPhone", hboxContactPhone,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hboxContactPhone);
gtk_box_pack_start (GTK_BOX (vboxContactFields), hboxContactPhone, TRUE, TRUE, 0);

entryContactPhone = gtk_entry_new_with_max_length (20);
gtk_widget_ref (entryContactPhone);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactPhone", entryContactPhone,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactPhone);
gtk_box_pack_start (GTK_BOX (hboxContactPhone), entryContactPhone, TRUE, TRUE, 0);
gtk_widget_set_usize (entryContactPhone, 50, -2);

hboxContactFax = gtk_hbox_new (FALSE, 0);
gtk_widget_ref (hboxContactFax);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hboxContactFax", hboxContactFax,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hboxContactFax);
gtk_box_pack_start (GTK_BOX (vboxContactFields), hboxContactFax, TRUE, TRUE, 0);

entryContactFax = gtk_entry_new_with_max_length (20);
gtk_widget_ref (entryContactFax);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactFax", entryContactFax,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactFax);
gtk_box_pack_start (GTK_BOX (hboxContactFax), entryContactFax, TRUE, TRUE, 0);
gtk_widget_set_usize (entryContactFax, 50, -2);

entryContactEmail = gtk_entry_new ();
gtk_widget_ref (entryContactEmail);
gtk_object_set_data_full (GTK_OBJECT (wMain), "entryContactEmail", entryContactEmail,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (entryContactEmail);
gtk_box_pack_start (GTK_BOX (vboxContactFields), entryContactEmail, FALSE, FALSE, 0);

vbuttonboxContact = gtk_vbutton_box_new ();
gtk_widget_ref (vbuttonboxContact);
gtk_object_set_data_full (GTK_OBJECT (wMain), "vbuttonboxContact", vbuttonboxContact,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (vbuttonboxContact);
gtk_box_pack_start (GTK_BOX (hboxContact), vbuttonboxContact, TRUE, TRUE, 0);
gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonboxContact), GTK_BUTTONBOX_SPREAD);
gtk_button_box_set_spacing (GTK_BUTTON_BOX (vbuttonboxContact), 0);

buttonContactSearch = gtk_button_new_with_label ("Search");
gtk_widget_ref (buttonContactSearch);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonContactSearch", buttonContactSearch,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonContactSearch);
gtk_container_add (GTK_CONTAINER (vbuttonboxContact), buttonContactSearch);
GTK_WIDGET_SET_FLAGS (buttonContactSearch, GTK_CAN_DEFAULT);

buttonContactNew = gtk_button_new_with_label ("New");
gtk_widget_ref (buttonContactNew);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonContactNew", buttonContactNew,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonContactNew);
gtk_container_add (GTK_CONTAINER (vbuttonboxContact), buttonContactNew);
GTK_WIDGET_SET_FLAGS (buttonContactNew, GTK_CAN_DEFAULT);

buttonContactUpdate = gtk_button_new_with_label ("Update");
gtk_widget_ref (buttonContactUpdate);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonContactUpdate", buttonContactUpdate,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonContactUpdate);
gtk_container_add (GTK_CONTAINER (vbuttonboxContact), buttonContactUpdate);
GTK_WIDGET_SET_FLAGS (buttonContactUpdate, GTK_CAN_DEFAULT);

buttonContactDelete = gtk_button_new_with_label ("Delete");
gtk_widget_ref (buttonContactDelete);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonContactDelete", buttonContactDelete,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonContactDelete);
gtk_container_add (GTK_CONTAINER (vbuttonboxContact), buttonContactDelete);
GTK_WIDGET_SET_FLAGS (buttonContactDelete, GTK_CAN_DEFAULT);

hbuttonboxMain = gtk_hbutton_box_new ();
gtk_widget_ref (hbuttonboxMain);
gtk_object_set_data_full (GTK_OBJECT (wMain), "hbuttonboxMain", hbuttonboxMain,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (hbuttonboxMain);
gtk_box_pack_start (GTK_BOX (vboxMain), hbuttonboxMain, FALSE, TRUE, 0);
gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonboxMain), GTK_BUTTONBOX_SPREAD);

buttonClear = gtk_button_new_with_label ("Clear");
gtk_widget_ref (buttonClear);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonClear", buttonClear,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonClear);
gtk_container_add (GTK_CONTAINER (hbuttonboxMain), buttonClear);
GTK_WIDGET_SET_FLAGS (buttonClear, GTK_CAN_DEFAULT);

buttonQuit = gtk_button_new_with_label ("Quit");
gtk_widget_ref (buttonQuit);
gtk_object_set_data_full (GTK_OBJECT (wMain), "buttonQuit", buttonQuit,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_widget_show (buttonQuit);
gtk_container_add (GTK_CONTAINER (hbuttonboxMain), buttonQuit);
GTK_WIDGET_SET_FLAGS (buttonQuit, GTK_CAN_DEFAULT);

gtk_signal_connect (GTK_OBJECT (wMain), "delete_event",
                      GTK_SIGNAL_FUNC (on_wMain_delete_event),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonContactSearch), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonContactSearch_pressed),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonContactNew), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonContactNew_pressed),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonContactUpdate), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonContactUpdate_pressed),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonContactDelete), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonContactDelete_pressed),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonClear), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonClear_pressed),
                      NULL);
gtk_signal_connect (GTK_OBJECT (buttonQuit), "pressed",
                      GTK_SIGNAL_FUNC (on_buttonQuit_pressed),
                      NULL);

tcontact = dbutil_table_new(TABLE_CONTACT);
fcontactid = dbutil_field_new(entryContactId, "id");
dbutil_field_set_flags(fcontactid, DBF_PRIMARYKEY|DBF_INT);
dbutil_table_add_field(tcontact, fcontactid);

fcontactname = dbutil_field_new(entryContactName, "name");
dbutil_field_set_flags(fcontactname, DBF_CHAR|DBF_SQLUSELIKE);
dbutil_table_add_field(tcontact, fcontactname);

fcontactaddr1 = dbutil_field_new(entryContactAddr1, "addr1");
dbutil_field_set_flags(fcontactaddr1, DBF_CHAR|DBF_SQLUSELIKE);
dbutil_table_add_field(tcontact, fcontactaddr1);

fcontactaddr2 = dbutil_field_new(entryContactAddr2, "addr2");
dbutil_field_set_flags(fcontactaddr2, DBF_CHAR|DBF_SQLUSELIKE);
dbutil_table_add_field(tcontact, fcontactaddr2);

fcontactcity = dbutil_field_new(entryContactCity, "city");
dbutil_field_set_flags(fcontactcity, DBF_CHAR|DBF_SQLUSELIKE);
dbutil_table_add_field(tcontact, fcontactcity);

fcontactstate = dbutil_field_new(entryContactState, "state");
dbutil_field_set_flags(fcontactstate, DBF_CHAR);
dbutil_table_add_field(tcontact, fcontactstate);

fcontactzip = dbutil_field_new(entryContactZip, "zip");
dbutil_field_set_flags(fcontactzip, DBF_CHAR);
dbutil_table_add_field(tcontact, fcontactzip);

fcontactphone = dbutil_field_new(entryContactPhone, "phone");
dbutil_field_set_flags(fcontactphone, DBF_CHAR);
dbutil_table_add_field(tcontact, fcontactphone);

fcontactfax = dbutil_field_new(entryContactFax, "fax");
dbutil_field_set_flags(fcontactfax, DBF_CHAR);
dbutil_table_add_field(tcontact, fcontactfax);

fcontactemail = dbutil_field_new(entryContactEmail, "email");
dbutil_field_set_flags(fcontactemail, DBF_CHAR|DBF_SQLUSELIKE);
dbutil_table_add_field(tcontact, fcontactemail);

tscontact = dbutil_tableset_new("tscontact");
dbutil_tableset_add_table(tscontact, tcontact);
rscontact = dbutil_relationset_new("rscontact", tscontact);

return wMain;
}

gboolean
on_wMain_delete_event                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
gtk_main_quit();
return FALSE;
}


void
on_buttonContactSearch_pressed         (GtkButton       *button,
                                        gpointer         user_data)
{
GString *sqlstr;
dbutil_gtk_table_sync_with_gui(tcontact);
sqlstr = dbutil_relationset_select(rscontact, TABLE_CONTACT, FALSE);
printf("Search: sql = '%s'\n", sqlstr->str);
g_string_free(sqlstr, TRUE);
}


void
on_buttonContactNew_pressed            (GtkButton       *button,
                                        gpointer         user_data)
{
GString *sqlstr;
dbutil_gtk_table_sync_with_gui(tcontact);
sqlstr = dbutil_relationset_insert(rscontact, TABLE_CONTACT);
printf("New: sql = '%s'\n", sqlstr->str);
g_string_free(sqlstr, TRUE);
}


void
on_buttonContactUpdate_pressed         (GtkButton       *button,
                                        gpointer         user_data)
{
GString *sqlstr;
dbutil_gtk_table_sync_with_gui(tcontact);
sqlstr = dbutil_relationset_update(rscontact, TABLE_CONTACT);
printf("Update: sql = '%s'\n", sqlstr->str);
g_string_free(sqlstr, TRUE);
}


void
on_buttonContactDelete_pressed         (GtkButton       *button,
                                        gpointer         user_data)
{
GString *sqlstr;
dbutil_gtk_table_sync_with_gui(tcontact);
sqlstr = dbutil_relationset_delete(rscontact, TABLE_CONTACT);
printf("Delete: sql = '%s'\n", sqlstr->str);
g_string_free(sqlstr, TRUE);
}


void
on_buttonClear_pressed                 (GtkButton       *button,
                                        gpointer         user_data)
{
dbutil_gtk_table_clear_fields(tcontact);
}


void
on_buttonQuit_pressed                  (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_main_quit();
}

